import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

@SuppressWarnings("serial")
public class CalendarReaderButton extends JPanel implements ActionListener
{
	private static final ImageIcon REMOVE_IMAGE = new ImageIcon("./images/remove.png");
	private static final ImageIcon HIDE_IMAGE = new ImageIcon("./images/hide.png");
	private static final ImageIcon SHOW_IMAGE = new ImageIcon("./images/show.png");
	
	// this is the invoice entry object represented by this InvoiceEntryButton
	private CalendarReader calendar;
	
	// this button will remove the Calendar from the list
	private JButton removeCalendar;
	
	// this button will make the calendar['s invoice entries] visible or invisible
	private JButton makeVisible;
	private boolean isVisible;
	
	// the title of the Calendar
	private JLabel title;
	
	public CalendarReaderButton(CalendarReader cr)
	{		
		// store the invoice entry this button represents
		this.calendar = cr;
		
		// store the title of the Calendar into "title" JLabel
		title = new JLabel(calendar.getTitle());
		
		// create the remove marker
		removeCalendar = new JButton(REMOVE_IMAGE);
		removeCalendar.setToolTipText("Remove this Calendar?");
		removeCalendar.addActionListener(this);
		removeCalendar.setContentAreaFilled(false);
		
		// create the hide/visible marker with default visible
		makeVisible = new JButton(SHOW_IMAGE);
		makeVisible.setToolTipText("Toggle this Calendar's visiblity");
		makeVisible.addActionListener(this);
		makeVisible.setContentAreaFilled(false);
		
		isVisible = true;
		
		JPanel controls = new JPanel();
		controls.setLayout(new GridLayout(1,2));
		((GridLayout)controls.getLayout()).setHgap(2);
		controls.add(makeVisible);
		controls.add(removeCalendar);
		
		// create the layout and add the important items to their necessary areas
		this.setLayout(new BorderLayout());
		this.add(title, BorderLayout.CENTER);
		this.add(controls, BorderLayout.EAST);
		
		// set the maximum size
	    this.setMaximumSize(new Dimension(1000,25));
	    
		// make the panel visible
		this.setVisible(true);
	}
	
	// what to do when this button is clicked
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		// get the button that was clicked
		JButton button = (JButton)e.getSource();
		
		// if the button that was clicked was the removeCalendar Button
		if(button.equals(removeCalendar))
		{
			try 
			{
				// if they confirm that they would like to remove this calendar
				if(JOptionPane.showConfirmDialog(null, "Are you sure you would like to remove this Calendar?") == JOptionPane.YES_OPTION)
				{
					// mark that the calendar should be removed
					InvoiceCreatorGUI.getInstance().removeCalendar(getCalendar());
				}
			} 
			catch (DataNotLoaded e1) 
			{
				// literally though, this should probably never happen...
				JOptionPane.showMessageDialog(null, "Unable to remove this Calendar at this time.");
				e1.printStackTrace();
			}
		}
		// if the button that was clicked was the makeVisible button
		else if(button.equals(makeVisible))
		{
			// toggle the visibility
			isVisible = !isVisible;
			
			// if the calendar is NOW visible
			if(isVisible)
			{
				// change the icon
				makeVisible.setIcon(SHOW_IMAGE);
			}
			// if the calendar is NOW NOT visible
			else
			{
				// change the icon
				makeVisible.setIcon(HIDE_IMAGE);
			}
			
			// call the method with the GUI to display the change for every invoice entry
			InvoiceCreatorGUI.getInstance().makeVisible(getCalendar(), isVisible);
		}
	}
	
	// returns the InvoiceEntry object this InvoiceEntryButton represents
	public CalendarReader getCalendar()
	{
		return calendar;
	}
}
