import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextArea;

@SuppressWarnings("serial")
public class InvoiceEntryButton extends JPanel implements ActionListener, Comparable
{
	private static final ImageIcon UNSELECTED_IMAGE = new ImageIcon("./images/unselected.png");
	private static final ImageIcon SELECTED_IMAGE = new ImageIcon("./images/selected.png");
	
	private static final Color WHITE = new Color(255,255,255);
	private static final Color GREEN = new Color(180,255,180);
	private static final Color GREY = new Color(200,200,200);
	
	// this is the invoice entry object represented by this InvoiceEntryButton
	private InvoiceEntry entry;
	
	// this button will toggle this InvoiceEntry's selection for print
	private JButton toggleSelection;
	private boolean isSelected;
	
	// this text area holds an <TODO> editable </TODO> account of the information to print
	private JTextArea content;
	
	public InvoiceEntryButton(InvoiceEntry ie)
	{
		// store the invoice entry this button represents
		this.entry = ie;
		
		// create the content description pane
		content = new JTextArea();
		content.setWrapStyleWord(true);
		
		//content.setToolTipText("Stores this entry's descriptive content");
		content.setLineWrap(true);
		content.setWrapStyleWord(true);
		content.setEditable(false);
		
		// create the selection marker with default unselected
		toggleSelection = new JButton();
		setSelection(false);
		toggleSelection.setToolTipText("Marks this entry to be included in the printed Invoice");
		toggleSelection.addActionListener(this);
		
		// default set the selected status to false
		isSelected = false;
		
		// create the layout and add the important items to their necessary areas
		this.setLayout(new BorderLayout());
		this.add(content, BorderLayout.CENTER);
		this.add(toggleSelection, BorderLayout.EAST);
		
		toggleSelection.setContentAreaFilled(false);
		content.setOpaque(false);
		
		this.setBackground(WHITE);
		
		// make the panel visible
		this.setVisible(true);
	}
	
	// what to do when this button is clicked
	@Override
	public void actionPerformed(ActionEvent e) 
	{
		// mark that the invoice's selection should be toggled
		setSelection(!isSelected);
		if(isSelected)
		{
			InvoiceCreatorGUI.getInstance().addHours(this.getEntry());
		}
		else
		{
			InvoiceCreatorGUI.getInstance().subHours(this.getEntry());
		}
	}
	
	// this function set the state of the selection to print and makes the visual changes to
	// make this change visible to the user
	private void setSelection(boolean selected)
	{
		isSelected = selected;
		
		// if the entry is now selected
		if(selected)
		{
			// change the selection icon to SELECTED
			toggleSelection.setIcon(SELECTED_IMAGE);
			this.setBackground(GREEN);
		}
		// if the entry is now NOT selected
		else
		{
			// change the selection icon UNSELECTED
			toggleSelection.setIcon(UNSELECTED_IMAGE);
			this.setBackground(WHITE);
		}
		
		this.setVisible(true);
	}
	
	// returns the state of the selection
	public boolean isSelected()
	{
		// return the state of the selection
		return isSelected;
	}
	
	// returns the InvoiceEntry object this InvoiceEntryButton represents
	public InvoiceEntry getEntry()
	{
		return entry;
	}
	
	// InvoiceEntryButtons are compared to one another using the functionality of
	// the InvoiceEntry they represent
	public int compareTo(Object obj2) 
	{
		// cast the object as an InvoiceEntryButton
		InvoiceEntryButton other = (InvoiceEntryButton) obj2;
		
		// use the InvoiceEntry's built in comparison method
		int retVal = this.getEntry().compareTo(other.getEntry());
		
		// return the result
		return retVal;
	}
	
	// this function takes an InvoiceEntryFormat and forwards it onto this object's
	// invoice entry object to be re-printed with the correct filters and the result
	// saved into the content textarea
	public void refreshText(InvoiceEntryFormat ie)
	{
		content.setText(entry.print(ie));
	}
	
	// this function sets the visibility of the invoice entry depending on the flag parameter
	public void makeVisible(boolean flag)
	{
		// if we are now visible,
		if(flag)
		{
			// we set the imagery back to how it should be when visible
			toggleSelection.setEnabled(true);
			setSelection(isSelected());
		}
		// if we are now INvisible
		else
		{
			// hide the selection icon and change the background
			//toggleSelection.setIcon(HIDDEN_IMAGE);
			toggleSelection.setEnabled(false);
			this.setBackground(GREY);
		}
	}
}
